unit MyCMD;

interface

uses
  Windows, Classes, Graphics, SysUtils, Dialogs, Math, Controls, Litecad, Events;

const
  CMD_SKETCH = LC_CMD_CUSTOM + 1;
  CMD_FLIP   = LC_CMD_CUSTOM + 2;
  CMD_ARROW  = LC_CMD_CUSTOM + 3;
  CMD_WALL   = LC_CMD_CUSTOM + 4;
  CMD_ID     = LC_CMD_CUSTOM + 5;

//----------------------------------------------
function  StrToPWChar(Str : String) : PWideChar;

//---------------------------------------------------
procedure CmdSketch_Start    (hLcCmd, Prm : Integer);
procedure CmdSketch_Finish   (hLcCmd : Integer);
procedure CmdSketch_MouseDown(hLcCmd, Button, Flags, Xwin, Ywin : Integer; X, Y : Double);
procedure CmdSketch_MouseUp  (hLcCmd, Button, Flags, Xwin, Ywin : Integer; X, Y : Double);
procedure CmdSketch_MouseMove(hLcCmd, hDC, Flags, Xwin, Ywin : Integer; X, Y : Double);
//---------------------------------------------------
procedure CmdFlip_Start      (hLcCmd, Prm : Integer);
procedure CmdFlip_Finish     (hLcCmd : Integer);
procedure CmdFlip_MouseDown  (hLcCmd, Button, Flags, Xwin, Ywin : Integer; X, Y : Double);
procedure CmdFlip_MouseUp    (hLcCmd, Button, Flags, Xwin, Ywin : Integer; X, Y : Double);
procedure CmdFlip_MouseMove  (hLcCmd, hDC, Flags, Xwin, Ywin : Integer; X, Y : Double);
//---------------------------------------------------
procedure CmdArrow_Start     (hLcCmd, Prm : Integer);
procedure CmdArrow_Finish    (hLcCmd : Integer);
procedure CmdArrow_MouseDown (hLcCmd, Button, Flags, Xwin, Ywin : Integer; X, Y : Double);
procedure CmdArrow_MouseUp   (hLcCmd, Button, Flags, Xwin, Ywin : Integer; X, Y : Double);
procedure CmdArrow_MouseMove (hLcCmd, hDC, Flags, Xwin, Ywin : Integer; X, Y : Double);
//---------------------------------------------------
procedure CmdWall_Start      (hLcCmd, Prm : Integer);
procedure CmdWall_Finish     (hLcCmd : Integer);
procedure CmdWall_MouseDown  (hLcCmd, Button, Flags, Xwin, Ywin : Integer; X, Y : Double);
procedure CmdWall_MouseUp    (hLcCmd, Button, Flags, Xwin, Ywin : Integer; X, Y : Double);
procedure CmdWall_MouseMove  (hLcCmd, hDC, Flags, Xwin, Ywin : Integer; X, Y : Double);

//--------------------------------------------------
procedure CmdID_Start       (hLcCmd, Prm : Integer);
procedure CmdID_Finish      (hLcCmd : Integer);
procedure CmdID_MouseDown   (hLcCmd, Button, Flags, Xwin, Ywin : Integer; X, Y : Double);
procedure CmdID_MouseUp     (hLcCmd, Button, Flags, Xwin, Ywin : Integer; X, Y : Double);
procedure CmdID_MouseMove   (hLcCmd, hDC, Flags, Xwin, Ywin : Integer; X, Y : Double);

implementation

//----------------------------------------------
function  StrToPWChar(Str : String) : PWideChar;
var
  WStr : WideString;
begin
  WStr   := Str;
  result := Addr(WStr[1]);
end;

//-----------------------------------------------
procedure CmdSketch_Start (hLcCmd, Prm : Integer);
var
  bOSnapOn, OSnapMode : Integer;
begin
  lcPropPutInt( hLcCmd, LC_PROP_CMD_STEP, 1 );
  // save current snap mode
  if lcPropGetBool( hLcCmd, LC_PROP_CMD_OSNAP ) = 1 then
    bOSnapOn := 1
  else
    bOSnapOn := 0;

  OSnapMode := lcPropGetInt( hLcCmd, LC_PROP_CMD_OSNAP );
  lcPropPutInt( hLcCmd, LC_PROP_CMD_INT0, bOSnapOn );
  lcPropPutInt( hLcCmd, LC_PROP_CMD_INT1, OSnapMode );
  // set snap mode for the command
  lcPropPutBool( hLcCmd, LC_PROP_CMD_OSNAP, 1 );
  lcPropPutInt( hLcCmd, LC_PROP_CMD_OSNAP, LC_SNAP_ENDPOINT );
  lcCmdPrompt ( hLcCmd, 'Input first point:' );

end;

//-----------------------------------------
procedure CmdSketch_Finish (hLcCmd : Integer);
var
  bOSnapOn, OSnapMode : Integer;
begin
  // restore original snap modes
  bOSnapOn  := lcPropGetInt( hLcCmd, LC_PROP_CMD_INT0 );
  OSnapMode := lcPropGetInt( hLcCmd, LC_PROP_CMD_INT1 );
  if bOSnapOn = 1 then
    lcPropPutBool( hLcCmd, LC_PROP_CMD_OSNAP, 1 )
  else
    lcPropPutBool( hLcCmd, LC_PROP_CMD_OSNAP, 0 );

  lcPropPutInt( hLcCmd, LC_PROP_CMD_OSNAP, OSnapMode );
end;

//-----------------------------------------------
procedure CmdSketch_MouseDown (hLcCmd, Button, Flags, Xwin, Ywin : Integer; X, Y : Double);
var
  hBlock, hPline : Integer;
begin
  case Button of
    LC_LBUTTON :
    begin
      hBlock := lcPropGetHandle( hLcCmd, LC_PROP_CMD_BLOCK );
      hPline := lcBlockAddPolyline( hBlock, 0, 0, 0 );
      lcPropPutHandle( hLcCmd, LC_PROP_CMD_HAND0, hPline );  // polyline
      lcPropPutInt( hLcCmd, LC_PROP_CMD_INT0, 1 );   // draw
    end;
    LC_RBUTTON : lcCmdExit();  // exit command
  end;
end;

//---------------------------------------------------------------------------------------
procedure CmdSketch_MouseUp (hLcCmd, Button, Flags, Xwin, Ywin : Integer; X, Y : Double);
begin
  if Button = LC_LBUTTON then
    lcPropPutInt( hLcCmd, LC_PROP_CMD_INT0, 0 );  // do not draw
end;

//----------------------------------------------------------------------------------
procedure CmdSketch_MouseMove (hLcCmd, hDC, Flags, Xwin, Ywin : Integer; X, Y : Double);
var
  hPline, hLcWnd, bDraw : Integer;
begin

  bDraw := lcPropGetInt( hLcCmd, LC_PROP_CMD_INT0 );
  if bDraw = 1 then begin
    hPline := lcPropGetHandle( hLcCmd, LC_PROP_CMD_HAND0 );
    lcPlineAddVer( hPline, 0, X, Y );
    lcCmdRegen( hLcCmd, hPline );
    hLcWnd := lcPropGetHandle( hLcCmd, LC_PROP_CMD_LCWND );
    lcWndRedraw( hLcWnd );
  end;
end;

//-----------------------------------------------
procedure CmdFlip_Start  (hLcCmd, Prm : Integer);
begin
  // display a text in the command line
  lcCmdPrompt( hLcCmd, 'Input center point:' );
end;

procedure CmdFlip_Finish (hLcCmd : Integer);
begin
  //
end;

procedure CmdFlip_MouseDown (hLcCmd, Button, Flags, Xwin, Ywin : Integer; X, Y : Double);
var
  hBlock, hLcWnd : Integer;
begin
  hBlock := lcPropGetHandle( hLcCmd, LC_PROP_CMD_BLOCK );
  // rotate selected objects 90 degrees around point X,Y
  lcBlockSelRotate( hBlock, X, Y, 90*LC_DEG_TO_RAD, 0, 0 );
  lcCmdRegen( hLcCmd, 0 );
  hLcWnd := lcPropGetHandle( hLcCmd, LC_PROP_CMD_LCWND );
  lcWndRedraw( hLcWnd );
  lcCmdExit();
end;

procedure CmdFlip_MouseUp (hLcCmd, Button, Flags, Xwin, Ywin : Integer; X, Y : Double);
begin
  //
end;

procedure CmdFlip_MouseMove (hLcCmd, hDC, Flags, Xwin, Ywin : Integer; X, Y : Double);
begin
  // display a text near cursor
  lcCmdPrompt( hLcCmd, 'Input center point:' );
end;

//----------------------------------------------------------------------
function AddArrow (hLcCmd : Integer; x0, y0, x1, y1 : Double) : Boolean;
var
  hBlock, hPline       : Integer;
  Len, x, y, dx, dy, w : Double;
begin

  dx  := x1 - x0;
  dy  := y1 - y0;
  Len := hypot(dx, dy);
  result := false;
  if Len > 0.0 then begin
    x := x0 + (dx * 0.8);
    y := y0 + (dy * 0.8);
    w := Len * 0.02;
    hBlock := lcPropGetHandle( hLcCmd, LC_PROP_CMD_BLOCK );
    hPline := lcBlockAddPolyline( hBlock, 0, 0, 0 );
    lcPlineAddVer2( hPline, 0, x0, y0, 0, w, w );
    lcPlineAddVer2( hPline, 0, x, y, 0, w*4.0, 0. );
    lcPlineAddVer2( hPline, 0, x1, y1, 0, 0.0, 0.0 );
    lcCmdRegen( hLcCmd, hPline );
    result := true;
  end;

end;

procedure CmdArrow_Start(hLcCmd, Prm : Integer);
var
  bOSnapOn, OSnapMode : Integer;
begin

  lcPropPutInt( hLcCmd, LC_PROP_CMD_STEP, 1 );
  // save current snap mode
  if lcPropGetBool( hLcCmd, LC_PROP_CMD_OSNAP ) = 1 then
    bOSnapOn := 1
  else
    bOSnapOn := 0;

  OSnapMode := lcPropGetInt( hLcCmd, LC_PROP_CMD_OSNAP );
  lcPropPutInt( hLcCmd, LC_PROP_CMD_INT0, bOSnapOn );
  lcPropPutInt( hLcCmd, LC_PROP_CMD_INT1, OSnapMode );
  // set snap mode for the command
  lcPropPutBool( hLcCmd, LC_PROP_CMD_OSNAP, 1 );
  lcPropPutInt( hLcCmd, LC_PROP_CMD_OSNAP, LC_SNAP_ENDPOINT );
  lcCmdPrompt( hLcCmd, 'Input first point:' );
  
end;

procedure CmdArrow_Finish    (hLcCmd : Integer);
var
  bOSnapOn, OSnapMode : Integer;
begin

  // restore original snap modes
  bOSnapOn  := lcPropGetInt( hLcCmd, LC_PROP_CMD_INT0 );
  OSnapMode := lcPropGetInt( hLcCmd, LC_PROP_CMD_INT1 );
  if bOSnapOn = 1 then
    lcPropPutBool( hLcCmd, LC_PROP_CMD_OSNAP, 1 )
  else
    lcPropPutBool( hLcCmd, LC_PROP_CMD_OSNAP, 0 );

  lcPropPutInt( hLcCmd, LC_PROP_CMD_OSNAP, OSnapMode );

end;

procedure CmdArrow_MouseDown (hLcCmd, Button, Flags, Xwin, Ywin : Integer; X, Y : Double);
var
  X0, Y0 : Double;
  Step   : Integer;
begin
  if Button = LC_LBUTTON then
    begin
      Step := lcPropGetInt( hLcCmd, LC_PROP_CMD_STEP );
      case Step of
        1 :
        begin
          lcPropPutInt( hLcCmd, LC_PROP_CMD_STEP, 2 );
          lcPropPutFloat( hLcCmd, LC_PROP_CMD_FLOAT0, X );
          lcPropPutFloat( hLcCmd, LC_PROP_CMD_FLOAT1, Y );
          lcPropPutInt( hLcCmd, LC_PROP_CMD_OSNAP, LC_SNAP_MIDPOINT );
          lcCmdPrompt( hLcCmd, 'Input second point:' );
        end;
        2 :
        begin
          X0 := lcPropGetFloat( hLcCmd, LC_PROP_CMD_FLOAT0 );
          Y0 := lcPropGetFloat( hLcCmd, LC_PROP_CMD_FLOAT1 );
          if AddArrow( hLcCmd, X0, Y0, X, Y ) then
            begin
              lcPropPutInt( hLcCmd, LC_PROP_CMD_STEP, 1 );
              lcPropPutInt( hLcCmd, LC_PROP_CMD_OSNAP, LC_SNAP_ENDPOINT );
              lcCmdPrompt( hLcCmd, 'Input first point:' );
            end
          else
            lcCmdPrompt( hLcCmd, 'Input second point:' );
        end;
      end;
    end
  else begin
    if Button = LC_RBUTTON then
      lcCmdExit();  // exit command
  end;

end;

procedure CmdArrow_MouseUp   (hLcCmd, Button, Flags, Xwin, Ywin : Integer; X, Y : Double);
begin
  //
end;

procedure CmdArrow_MouseMove (hLcCmd, hDC, Flags, Xwin, Ywin : Integer; X, Y : Double);
var
  X0, Y0 : Double;
  Step   : Integer;
begin
  Step := lcPropGetInt( hLcCmd, LC_PROP_CMD_STEP );
  case Step of
    1: lcCmdPrompt( hLcCmd, 'First point' );
    2:
    begin
      lcCmdPrompt( hLcCmd, 'Second point' );
      X0 := lcPropGetFloat( hLcCmd, LC_PROP_CMD_FLOAT0 );
      Y0 := lcPropGetFloat( hLcCmd, LC_PROP_CMD_FLOAT1 );
      lcCmdDrawLine( hLcCmd, X0, Y0, X, Y );
    end;
  end;
end;

//---------------------------------------------------------------------
procedure CmdWall_Start (hLcCmd, Prm : Integer);
var
  hView, bOrtho : Integer;
begin
  hView := lcPropGetHandle( hLcCmd, LC_PROP_CMD_VIEW );
  // save current Ortho mode
  if lcPropGetBool( hView, LC_PROP_VIEW_ORTHO ) = 1 then
    bOrtho := 1
  else
    bOrtho := 0;

  lcPropPutInt( hLcCmd, LC_PROP_CMD_INT0, bOrtho );
  // set Ortho mode for the command
  lcPropPutBool( hView, LC_PROP_VIEW_ORTHO, 1 );
  // clear points buffer
  lcCmdClearPoints( hLcCmd );
end;

procedure CmdWall_Finish (hLcCmd : Integer);
var
  i, nPnts, bOrtho, hBlock, hPline, hView : Integer;
  x, y: Double;
begin

  nPnts := lcPropGetInt( hLcCmd, LC_PROP_CMD_NPOINTS );
  if nPnts >= 2 then begin
    hBlock := lcPropGetHandle( hLcCmd, LC_PROP_CMD_BLOCK );
    hPline := lcBlockAddPolyline( hBlock, 0, 0, 0 );
    for i := 0 to nPnts do begin
      lcCmdGetPoint( hLcCmd, i, @x, @y );
      lcPlineAddVer( hPline, 0, x, y );
    end;
    lcPropPutFloat( hPline, LC_PROP_PLINE_WIDTH, 2.0 );
    lcPropPutStr( hPline, LC_PROP_ENT_COLOR, '34' );
    lcCmdRegen( hLcCmd, hPline );
  end;
  // restore previous Ortho mode
  hView := lcPropGetHandle( hLcCmd, LC_PROP_CMD_VIEW );
  bOrtho := lcPropGetInt( hLcCmd, LC_PROP_CMD_INT0 );
  if bOrtho = 1 then
    lcPropPutBool( hView, LC_PROP_VIEW_ORTHO, 1 )
  else
    lcPropPutBool( hView, LC_PROP_VIEW_ORTHO, 0 );

end;

procedure CmdWall_MouseDown (hLcCmd, Button, Flags, Xwin, Ywin : Integer; X, Y : Double);
var
  nPnts : Integer;
begin

  if Button = LC_LBUTTON then
    begin
      nPnts := lcPropGetInt( hLcCmd, LC_PROP_CMD_NPOINTS );
      lcCmdAddPoint( hLcCmd, nPnts, X, Y );
      inc(nPnts);
      lcCmdSetBasePoint( hLcCmd, 1, X, Y );
    end
  else
    if Button = LC_RBUTTON then
      lcCmdExit();  // exit command

end;

procedure CmdWall_MouseUp (hLcCmd, Button, Flags, Xwin, Ywin : Integer; X, Y : Double);
begin
  //
end;

procedure CmdWall_MouseMove (hLcCmd, hDC, Flags, Xwin, Ywin : Integer; X, Y : Double);
var
  i, nPnts : Integer;
  x0, y0, z0, x1, y1, z1 : Double;
begin

  nPnts := lcPropGetInt( hLcCmd, LC_PROP_CMD_NPOINTS );
  if nPnts = 0 then
    lcCmdPrompt( hLcCmd, 'First point' )
  else begin
    lcCmdPrompt( hLcCmd, 'Next point' );
    if nPnts >= 2 then begin
      lcCmdGetPoint( hLcCmd, 0, @x0, @y0 );
      for i := 1 to nPnts do begin
        lcCmdGetPoint( hLcCmd, 0, @x1, @y1 );
        lcCmdDrawLine( hLcCmd, x0, y0, x1, y1 );
        x0 := x1;
        y0 := y1;
      end;
    end;
    lcCmdGetPoint( hLcCmd, nPnts-1, @x0, @y0 );
    lcCmdDrawLine( hLcCmd, x0, y0, X, Y );
  end;

end;

//--------------------------------------------
procedure CmdID_Start (hLcCmd, Prm : Integer);
begin
  lcCmdPrompt( hLcCmd, 'Specify point:' );
end;

procedure CmdID_Finish (hLcCmd : Integer);
begin
  //
end;

procedure CmdID_MouseDown (hLcCmd, Button, Flags, Xwin, Ywin : Integer; X, Y : Double);
var
  S          : String;
  SX, SY, SZ : string[32];
begin
  if Button = LC_LBUTTON then begin
    lcCmdGetPoint( hLcCmd, 0, @X, @Y );
    SX := FormatFloat('0.000', X);
    SY := FormatFloat('0.000', Y);
    S  := 'X = ' + SX + '   Y = ' + SY;
    lcCmdPrompt( hLcCmd, StrToPWChar(S) );
    lcCmdExit();  // exit command
  end;
end;

procedure CmdID_MouseUp (hLcCmd, Button, Flags, Xwin, Ywin : Integer; X, Y : Double);
begin
  //
end;

procedure CmdID_MouseMove (hLcCmd, hDC, Flags, Xwin, Ywin : Integer; X, Y : Double);
begin
  //
end;

end.

